#
# solver.rb
#
# dependency solver
#
# solves install or remove requests by looking at dependencies
# and generates a solving graph consisting of nodes and edges
#
require 'zypptools/lib/arch'
require 'zypptools/graph/dependencies'
require 'zypptools/graph/graph'
require 'zypptools/graph/reason'

class Solver
  attr_reader :solution
  attr_accessor :merge_devel

  def initialize( depgraph, arch )
    @world = depgraph
    @arch = arch
    @solution = Graph.new
    @tags = Hash.new
    @resolvables = Hash.new
  end

  #
  # install
  #  item = tag (string), resolvable, or dependency
  #  kind = kind of item (if string or dependency)
  #  reason = reason for install request
  #  depth = remaining depth (starting from initial user install request)
  #
  # returns
  #  resolvable matching the install request
  #  true if depth too small
  #  nil on error
  #

  def install( item, kind, reason, depth )
STDERR.puts "Solver.install #{kind}:#{item}, #{reason}, #{depth}" if $debug && $debug > 1

    return true if depth < 0

    # item is dependency

    if item.class == Dependency
STDERR.puts "Solver.install dependency '#{kind}:#{item}'" if $debug
      # FIXME: check actual dependency, including operator
      return install( item.name, kind, reason, depth )

    # item is string

    elsif item.class == String
      if @merge_devel
	if item[-6,6] == "-devel"
	  item = item[0..-7]
	end
      end

      # return if this tag is already known
      return @tags[item] if @tags[item]

STDERR.puts "Solver.install tag '#{kind}:#{item}'" if $debug

      # check the world for matching providers

      providers = @world.providers( kind, item )

      if providers.nil? || providers.empty?	# no providers for dependency
	STDERR.puts "No providers for '#{kind}:#{item}'" if reason.kind == :requires
	return nil
      end

      # now add providers to graph and trigger install request for all of them

      providers.each do |entry|
	next unless @arch.compat?( entry.resolvable.architecture )
STDERR.puts "Solver.install: #{entry.to_s} provides #{kind}:#{item}" if $debug

	# add providing resolvable as node to graph

	node = @solution.add( entry.resolvable )
STDERR.puts "Solver.install: Node #{node}" if $debug

	# create edge from reason.node to (new) node

	edge = Edge.create( item, reason.requestor, node, reason ) if reason
	node.incoming << edge if edge
	reason.requestor.outgoing << edge if edge

STDERR.puts "Solver.install: Edge #{edge}" if $debug

#	@tags[item] = true

	# install provider

	@tags[item] = install( entry.resolvable, entry.resolvable.kind, reason, depth )

	return @tags[item]

      end

    # item is resolvable

    elsif item.kind_of?( Resolvable )
STDERR.puts "Solver.install: Resolvable #{item}" if $debug

      # return if resolvable already known

      return @resolvables[item] if @resolvables[item]

      result = true

      # add resolvable as node to graph

      node = @solution.add( item )
STDERR.puts "Solver.install resolvable '#{node.to_s}'" if $debug
      @resolvables[item] = item

      # install all requirements

      item.requires.each do |dep|
	reason = Reason.new( :requires, dep, node )
	result = false unless install( dep, dep.refers, reason, depth - 1 )
      end if item.requires

      @resolvables[item] = nil unless result

      # install all recommends (ignoring result)

      item.recommends.each do |dep|
	reason = Reason.new( :recommends, dep, node )
	install( dep, dep.refers, reason, depth - 1 )
      end if item.recommends && result

#STDERR.puts "Solver.install: @resolvables[#{item}] = #{@resolvables[item]}"
      STDERR.puts "Resolvable #{item.to_s} is not installable" unless result
      return @resolvables[item]

    # everything else is an error

    else
      STDERR.puts "Install can't handle class #{item.class}"
      return nil
    end

    raise "You shouldn't be here."
  end

  def remove( item )
  end

end
