require 'pathname'
require 'lib/package'
require "rexml/document"

include REXML

#
# This class parses a channel in Helix
# XML format
#
class HelixChannelParser
  attr_reader :packages, :patterns, :patches, :products
  
  #
  # p.HelixChannelParser.new( "channel.xml" )
  # p.packages
  #
  def initialize( filename )
    @packages = Hash.new
    @patches = Hash.new
    @doc = Document.new(File.new(filename))
   
    parse_channels
  end
    
  private
  def parse_channels
    @doc.elements.each("channel/subchannel/*") do |element| 
      if element.name == 'package'
        parse_package_element element
      end
    end
  end
    
  def parse_package_element(element)
    #def initialize( name, epoch = nil, version = nil, release = nil, architecture = nil )
    pkg = Package.new("")
    element.elements.each do |property_element|
      case property_element.name.to_sym

      when :channel, :section, :filename, :filesize, :installedsize, :md5sum, :importance, :description
        next
      when :history
        parse_history_element( pkg, property_element)
      when :requires, :provides, :conflicts, :obsoletes, :suggests, :enhances, :supplements, :recommends, :freshens
        capset = parse_capset_element( property_element )
#       puts "*****\n#{capset.inspect.to_s}\n******"
        method_name = property_element.name
        deps = pkg.send( method_name )
        begin
          if ( deps.nil? )
            deps = Array.new
          end
          deps += capset
#puts "++++++\n#{deps.inspect.to_s}\n++++++"
          next
        rescue NoMethodError=> boom
          puts "Missing method in capset: #{method_name}"
          exit
        rescue Exception => boom
          puts boom
          exit
        end
      else
        pkg.send( property_element.name + '=', property_element.text )
      end
    end
    pkg.add_self_provide
    @packages[pkg.to_s]= pkg
  end
  
  # not yet used
#   def parse_patch_element(element)
#     patch = Patch.new("")
#     element.elements.each do |property_element|
#       case property_element.name.to_sym
#       when :arch
#         self.architecture = property_element.text
#       when :requires, :provides, :conflicts, :obsoletes, :suggests, :enhances, :supplements, :recommends, :freshens
#         capset = parse_capset_element( property_element )
#         puts "*****\n#{capset.inspect.to_s}\n******"
#         begin
#           if ( (patch.send( property_element.name )).nil? )
#             patch.send( property_element.name + "=", Array.new )
#           end
#           (patch.send( property_element.name )).send( "=",  capset)
#           next
#         rescue NoMethodError=> boom
#           puts property_element.name
#           exit
#         end
#       else
#         patch.send( property_element.name + '=', property_element.text )
#       end
#     end
#     @patches[patch.to_s] = patch
#   end  
  
  def parse_history_element( pkg, element )
    element.elements.each do |his_element|
      case his_element.name
        when 'update':
          parse_update_element( pkg, his_element)
      end
    end
  end
    
  def parse_update_element( pkg, element )
    element.elements.each do |up_element|
      #puts "set #{up_element.name} to #{up_element.text}"
      case up_element.name.to_sym
      when :channel, :section, :filename, :filesize, :installedsize, :md5sum, :importance, :description
        next
      when :hid
        next
      when :arch
        method_name = 'architecture'
      end
      method_name = up_element.name
      pkg.send( method_name + '=', up_element.text )
    end
  end
  
  def parse_capset_element( element )
    capset = Array.new
    element.elements.each do |dep_element|
      op = dep_element.attributes['op']
      (op = "==") if (op == "=")
      d = Dependency.new( :refers => dep_element.attributes['refers'], :name => dep_element.attributes['name'], :op => op, :pre => ( dep_element.attributes['pre'] == '1' ), :version => dep_element.attributes['version'], :release => dep_element.attributes['release'] )
      
#puts "+Dep \n#### #{d.inspect}\n##### #{capset.size} in #{element.name}"
      capset.push d
      return capset
    end
#puts "parsed #{capset.size} dependencies"
    return capset
  end
end

class Measure
  def initialize(name)
    start(name)
  end

  def start(name)
    @name = name
    @start = Time.now
    puts "#{name} start"
  end
    
  def elapsed
    e = Time.now - @start
    puts "#{@name} : #{e.to_f}"
  end
end

class HelixTransactionInstall
   attr_accessor :channel, :package
   def initialize( channel, package )
     @channel = channel
     @package = package
   end
   
   def to_s
      return "INSTALL #{@package} from #{@channel}"
   end
end
   
class HelixTransactionRequire
   attr_accessor :capability
   def initialize(capability)
    @capability = capability
   end
    
   def to_s
      return "REQUIRE #{@capability}"
   end
end
    
class HelixTestParser
  attr_accessor :channels, :transaction_install, :transaction_require

  def initialize( filename )
    @channels = Hash.new
    @transaction_install = Array.new
    @transaction_require = Array.new
    @basedir = Pathname.new(filename).dirname
    @doc = Document.new(File.new(filename))
   
    parse_test
  end

  def parse_test
    puts "Parse test"
    @doc.elements.each("test/*") do |element| 
      case element.name
      when 'setup'
        parse_setup element
      when 'trial'
        parse_trial element
      end
    end
  end
    
  def parse_setup(element)
    element.elements.each do |setup_element|
      case setup_element.name
      when 'setup'
        @arch = setup_element.attributes['arch']
      when 'system'
        add_repo( setup_element.attributes['file'], '@system' )
      when 'channel'
        add_repo( setup_element.attributes['file'], setup_element.attributes['name'] )
      end
    end
  end
  
  def parse_trial(element)
    element.elements.each do |trial_element|
      case trial_element.name
      when 'install'
        t = HelixTransactionInstall.new(trial_element.attributes['channel'], trial_element.attributes['package'])
        puts "Adding transaction #{t}"
        @transaction_install << t
      when 'addRequire'
        t = HelixTransactionRequire.new(trial_element.attributes['name'])
        puts "Adding transaction #{t}"
        @transaction_require << t
      end
    end
  end
    
    
  def add_repo( channel_file, reponame )
    if File.exists?(channel_file)
      @channels[reponame] = channel_file
      puts "Add channel #{channel_file} as #{reponame}"
    elsif File.exists?( (@basedir + channel_file).to_s )
       @channels[reponame] = (@basedir + channel_file).to_s
       puts "Add channel #{(@basedir + channel_file).to_s} as #{reponame}"
    else
      raise "Can't find #{channel_file} or #{@basedir + channel_file}"
    end
  end
end
    
if $0 == __FILE__
  if not File.exists?('packages.cache')
    m = Measure.new('parse')
    parser = HelixChannelParser.new(ARGV[0])
    puts "Read #{parser.packages.size}  packages"
    m.elapsed
    m.start('write cache')
    f = File.new("packages.cache", 'w')
    f.write(Marshal.dump(parser.packages))
    f.close
    m.elapsed
    packages = nil
  else
    m = Measure.new("read cache")
    f = File.new("packages.cache", 'r')
    packages = Marshal.load(f.read)
    puts "Read (cache) #{packages.size}  packages"
    m.elapsed
    puts packages.first.to_s
  end
end
