#
# yast source parser
#
# Parses files of a 'yast' source
# typically 'content', 'suse/setup/descr/packages', 'suse/setup/descr/*.pat'
#

require 'zypptools/input/tagparser'
require 'zypptools/input/contentparser'
require 'zypptools/lib/package'
require 'zypptools/lib/pattern'
require 'zypptools/lib/selection'
require 'zypptools/lib/product'
require 'zypptools/lib/arch'
require 'zlib'

class YaST
  #
  # parse_NEVRA - parse name, epoch, version, release, architecture from string
  #
  def YaST.parse_NEVRA( str )
    name,version,release,arch = str.split " "
    if version.include? ":"		# an epoch ?
      epoch,version = version.split ":"
    else
      epoch = nil
    end
    return name,epoch,version,release,arch
  end
end

class Resolvables < TagParser

  attr_reader :store, :arch

  def initialize filename, arch
    if arch.kind_of?( Arch )
      @arch = arch
    else
      @arch = Arch.new( arch )
    end
    @missing = Hash.new
    @store = Hash.new
    @current = nil
    super filename
  end

  def finish
    @store[@hashid] = @current if @current
    @current = nil
  end

  def Sum value
    @current.summary = value unless @lang
  end

  def Des value
    @current.description = value.join("\n") unless @lang
  end

  def Req value
    @current.deps :requires, value
  end

  def Prq value
    @current.deps :prerequires, value, :package, true
  end

  def Prv value
    @current.deps :provides, value
  end

  def Con value
    @current.deps :conflicts, value
  end

  def Obs value
    @current.deps :obsoletes, value
  end

  def Sup value
    @current.deps :supplements, value
  end

  def Enh value
    @current.deps :enhances, value
  end

  def Sug value
    @current.deps :suggests, value
  end

  def Rec value
    @current.deps :recommends, value
  end

  def method_missing meth, *args
    return if @missing.has_key? meth
#STDERR.puts "#{meth} is missing"
    @missing[meth] = true
  end

end

class Packages < Resolvables

  def Ver version
    raise "Invalid version" unless version == "2.0"
  end

  def Pkg value
    finish
    n,e,v,r,a = YaST.parse_NEVRA value
    if @arch.compat?( a )
#	STDERR.puts "#{value} is compatible with #{@arch.arch}"
	@current = Package.new( n, e, v, r, a )
	@current.add_self_provide
	@hashid = value
    else
	skip_until( "Pkg" )
    end
  end

  def Cks value
    @current.chk_type, @current.chk_sum = value.split " "
  end

  def Shr value
    if @store.has_key?( value )
      @current.shared = @store[value]
    else
      STDERR.puts "Shared target unknown #{value}"
    end
  end

  def Grp value
    @current.group = value
  end

  def Lic value
    @current.license = value
  end

  def Src value
    @current.srcrpm = value
  end

  def Tim value
    @current.buildtime = value
  end

  def Loc value
    @current.location = value
  end

  def Siz value
    @current.rpmsize, @current.installsize = value.split " "
  end

  def Aut value
    @current.authors = value
  end

end

class Patterns < Resolvables

  def Ver version
    raise "Invalid version" unless version == "5.0"
  end

  def Pat value
    finish
    n,e,v,r,a = YaST.parse_NEVRA value
    if @arch.compat?( a )
	@current = Pattern.new( n, e, v, r, a )
	@current.add_self_provide
	@hashid = value
    else
	skip_until( "Pat" )
    end
    @hashid = value
  end

  def Prq value
    @current.deps :requires, value, :package
  end

  def Psg value
    @current.deps :suggests, value, :package
  end

  def Prc value
    @current.deps :recommends, value, :package
  end

  def Fre value
    @current.deps :freshens, value, :pattern
  end

  def Cat value
    @current.category = value unless @lang
  end

  def Vis value
    @current.visible = value
  end

  def Ord value
    @current.order = value
  end

  def Ico value
    @current.icon = value
  end

  def Ext value
    # extends -- pattern display
  end

end

class Content < ContentParser

  attr_reader :store

  def initialize filename
    @store = Hash.new
    @current = Product.new ""
    super filename
  end

  def finish
	@store[@store.size] = @current
  end

  def product value
	@current.name = value
  end

  def defaultbase value
	@current.architecture = value
  end

  def requires value
	@current.requires = []
	value.split(" ").each do |v|
		kind, name = v.split(":")
		unless name
			name = kind
			kind = "package"
		end
		@current.requires << Dependency.new( name, kind.to_sym )
	end
  end

  def provides(value)
  end

  def obsoletes(value)
  end

  def label value
	@current.summary = value unless @subtag
  end

  def shortlabel value
	@current.shortlabel = value
  end

  def patterns value
	@current.patterns = value.split( " " )
  end

  def meta value
  end

  def key value
  end

  def extraurls value
  end

  def optionalurls value
  end

  def type value
  end

  def hash value
  end

  def method_missing meth, arg
	@current.send( meth.to_s + "=", arg )
  end
end

class Selections < Resolvables

  def Ver version
	#raise "Invalid version" unless version == "5.0"
  end

  def Sel value
	finish
    n,e,v,r,a = YaST.parse_NEVRA value
    if @arch.compat?( a )
		@current = Selection.new( n, e, v, r, a )
		@current.add_self_provide
		@hashid = value
    else
		skip_until( "Sel" )
    end
    @hashid = value
  end

  def Ins value
	@current.deps :requires, value, :package
  end

  def Psg value
    @current.deps :suggests, value, :package
  end

  def Prc value
    @current.deps :recommends, value, :package
  end

  def Fre value
    @current.deps :freshens, value, :selection
  end

  def Cat value
    @current.category = value unless @lang
  end

  def Vis value
    @current.visible = value
  end

  def Ord value
    @current.order = value
  end

  def Ico value
    @current.icon = value
  end

end

class YaST
    attr_reader :packages, :patterns, :patches, :products, :selections

    def initialize( dir, arch )
	raise "Not a directory '#{dir}'" unless File.directory?( dir )
	@dir = dir
	content = Content.new( dir + "/content")
	return unless content

	@products = content.store

	if @products.size > 1
		STDERR.puts "Multiple products not supported"
		return
	end
	dir += "/" + @products[0].descrdir

   if File.exist?(dir + "/packages")
	   @pkgs = Packages.new( dir + "/packages", arch )
   elsif File.exist?(dir + "/packages.gz")
      data = File.new("tmp_yast", modestring="w+")
      File.open(dir + "/packages.gz") do |f|
         gzip = Zlib::GzipReader.new(f)
         data << gzip.read
         gzip.close
      end
      data.close
      @pkgs = Packages.new( "tmp_yast", arch )
      File.delete("tmp_yast")
   else
      raise "Packages-File not found"        
   end
	@packages = @pkgs.store
	@patterns = Hash.new
	pname = dir + "/patterns"
	return unless File.exist?( pname )
	File.open( pname ).each { |p|
	    pat = Patterns.new( dir + "/" + p.chomp!, arch )
	    #@patterns = pat.store
		@patterns.merge! pat.store
	}
	@selections = Hash.new
	sname = dir + "/selections"
	return unless File.exist?( sname )
	File.open( sname ).each { |p|
		sel = Selections.new( dir + "/" + p.chomp!, arch )
		@selections.merge! sel.store
	}
    end

end
